#include "burnint.h"
// CPS - Draw

unsigned char CpsRecalcPal=0; // Flag - If it is 1, recalc the whole palette

int nCpsLcReg=0; // Address of layer controller register
int CpsLayEn[4]={0,0,0,0}; // bits for layer enable

static int DrawScroll1()
{
  int nOff=0,nScrX=0,nScrY=0;
  unsigned char *Find=NULL;

  // Draw Scroll 1
  nOff=*((unsigned short *)(CpsSaveReg+0x02)); nOff<<=8;

  // Get scroll coordinates
  nScrX=*((unsigned short *)(CpsSaveReg+0x0c)); // Scroll 1 X
  nScrY=*((unsigned short *)(CpsSaveReg+0x0e)); // Scroll 1 Y
  nScrX+=0x40; nScrY+=0x10;
  
  nOff&=0xffc000; Find=CpsFindGfxRam(nOff,0x4000); if (Find==NULL) return 1;
  CpsScr1Draw(Find,nScrX,nScrY);
  return 0;
}

static INLINE int DrawScroll2Init()
{
  int nScr2Off=0; int n=0;

  // Draw Scroll 2
  nScr2Off =*((unsigned short *)(CpsSaveReg+0x04)); nScr2Off<<=8;
  nScr2Off&=0xffc000; CpsrBase=CpsFindGfxRam(nScr2Off,0x4000);
  if (CpsrBase==NULL) return 1;

  // Get scroll coordinates
  nCpsrScrX=*((unsigned short *)(CpsSaveReg+0x10)); // Scroll 2 X
  nCpsrScrX+=0x40; nCpsrScrX&=0x3ff; 

  nCpsrScrY=*((unsigned short *)(CpsSaveReg+0x12)); // Scroll 2 Ytess
  nCpsrScrY+=0x10; nCpsrScrY&=0x3ff;

  // Get row scroll information
  n=*((unsigned short *)(CpsSaveReg+0x22));

  CpsrRows=NULL;
  if (n&1)
  {
    int nTab,nStart;
    // Find row scroll table:
    nTab=*((unsigned short *)(CpsSaveReg+0x08)); nTab<<=8;

    nTab&=0xfff800; // Vampire - Row scroll effect in VS screen background
    
    CpsrRows=(unsigned short *)CpsFindGfxRam(nTab,0x800);

    // Find start offset
    nStart=*((unsigned short *)(CpsSaveReg+0x20));
    nCpsrRowStart=nStart+16;
  }

  // Prepare Scroll 2
  CpsrPrepare();
  return 0;
}

static INLINE int DrawScroll2Exit()
{
  CpsrBase=NULL;
  nCpsrScrX=NULL;  nCpsrScrY=NULL;  CpsrRows=NULL;
  return 0;
}

static INLINE int DrawScroll2Do()
{
  if (CpsrBase==NULL) return 1;
  CpsrRender();
  return 0;
}

static int DrawScroll3()
{
  int nOff=0,nScrX=0,nScrY=0;
  unsigned char *Find=NULL;

  // Draw Scroll 3
  nOff=*((unsigned short *)(CpsSaveReg+0x06)); nOff<<=8;

  // Get scroll coordinates
  nScrX=*((unsigned short *)(CpsSaveReg+0x14)); // Scroll 3 X
  nScrY=*((unsigned short *)(CpsSaveReg+0x16)); // Scroll 3 Y
  nScrX+=0x40; nScrY+=0x10;
  
  nOff&=0xffc000; Find=CpsFindGfxRam(nOff,0x4000); if (Find==NULL) return 1;

  CpsScr3Draw(Find,nScrX,nScrY);
  return 0;
}

static void Layers(int LayerCont)
{
  int Draw[4]={-1,-1,-1,-1};
  int Prio[4]={0,0,0,0};
  int nSpriteTo=0; // We have drawn sprites up to this priority number minus one
  int nDrawMask=0;
  int i=0;

  nDrawMask=1; // Sprites always on
  // Get correct bits from Layer Controller
  if (LayerCont & CpsLayEn[1]) nDrawMask|=2;
  if (LayerCont & CpsLayEn[2]) nDrawMask|=4;
  if (LayerCont & CpsLayEn[3]) nDrawMask|=8;
  nDrawMask&=nBurnLayer;   // User choice of layers to display

  // Layer control:
  Draw[0]=(LayerCont>>12)&3; // top layer
  Draw[1]=(LayerCont>>10)&3;
  Draw[2]=(LayerCont>> 8)&3;
  Draw[3]=(LayerCont>> 6)&3; // bottom layer (most covered up)

  if (Cps==2)
  {
    int nLayPri=0;
    nLayPri=(CpsFrg[4]<<8) | CpsFrg[5]; // Layer priority register at word(400004)
    // Layer priority (if greator or equal to sprite priority, it covers them)
    Prio[0]=(nLayPri>> 4)&7;
    Prio[1]=(nLayPri>> 8)&7;
    Prio[2]=(nLayPri>>12)&7;
  }

  // Check for repeated layers and if there are any, the lower layer is omitted
#define CRP(a,b) if (Draw[a]==Draw[b]) Draw[b]=-1;
  CRP(0,1) CRP(0,2) CRP(0,3) CRP(1,2) CRP(1,3) CRP(2,3)
#undef CRP

  // prepare layer 2
  DrawScroll2Init();

  // draw layers, bottom -> top
  for (i=3;i>=0;i--)
  {
    int n=Draw[i]; // Find out which layer to draw

    if (Cps==2 && n>=1)
    {
      int nPri=0;
      // Find out the priority of this layer
      nPri=Prio[n-1];
      // Draw sprites up to this priority
      if (nPri<nSpriteTo) nSpriteTo=0; // go back for more sprites
      if (nDrawMask&1) CpsObjDraw(nSpriteTo,nPri);
      nSpriteTo=nPri+1;
    }

    if (Cps!=2 && n==0) { CpsObjDraw(0,7); } // CPS1 sprites

    // Then Draw the scroll layer on top
         if (n==1) { if (nDrawMask&2) DrawScroll1(); }
    else if (n==2) { if (nDrawMask&4) DrawScroll2Do(); }
    else if (n==3) { if (nDrawMask&8) DrawScroll3();   }
  }

  DrawScroll2Exit();

  if (Cps==2 && nSpriteTo<=7) { if (nDrawMask&1) CpsObjDraw(nSpriteTo,7); } // draw the remaining sprites
}

int CpsDraw()
{
  int LayerCont=0;

  CtvReady(); // Point to correct tile drawing functions
  BurnClearSize(384,224); // Because of vertical games
  // Update Palette
  CpsPalUpdate(CpsSavePal,CpsRecalcPal); // recalc whole palette if needed
  CpsRecalcPal=0;

  // Layer controller
  LayerCont=*((unsigned short *)(CpsSaveReg+nCpsLcReg));

  Layers(LayerCont);

  return 0;
}
