#include "burnint.h"
// CPS - Run

// Inputs:
unsigned char CpsReset=0;
unsigned char Cpi01A=0,Cpi01C=0,Cpi01E=0;

static int DrvReset()
{
  // Reset machine
  CpsEepromReset();

  SekOpen(0); SekReset();
  SekClose();

  ZetReset(); // Reset z80
  return 0;
}

int CpsRunInit()
{
  int nRet=0;
  SekInit(1); // Allocate 68000

  // Memory init
  nRet=CpsMemInit(); if (nRet!=0) return 1;
  CpsEepromBlank();
  CpsRwInit();

  // Graphics init
  nRet=CpsPalInit(); if (nRet!=0) return 1;
  nRet=CpsObjInit(); if (nRet!=0) return 1;

  // Sound init
  if (Cps==1) { nRet=PsndInit(); if (nRet!=0) return 1; }
  if (Cps==2) { nRet=QsndInit(); if (nRet!=0) return 1; }

  CpsEepromReset();
  DrvReset();
  return 0;
}

int CpsRunExit()
{
  // Sound exit
  if (Cps==2) QsndExit();
  if (Cps==1) PsndExit();

  // Graphics exit
  CpsObjExit();
  CpsPalExit();

  // Memory exit
  CpsRwExit();
  CpsMemExit();

  SekExit(); // Deallocate 68000

  return 0;
}

// nStart = 0-3, nCount=1-4
static INLINE void GetPalette(int nStart,int nCount)
{
  int nPal; unsigned char *Find;
  // Update Palette
  nPal=*((unsigned short *)(CpsReg+0x0a)); nPal<<=8;

  // Ghouls points to the wrong place on boot up I think:
  nPal&=0xfff800; Find=CpsFindGfxRam(nPal,0x1000);
  if (Find==NULL) return;
  memcpy(CpsSavePal+(nStart<<10), Find+(nStart<<10), nCount<<10);
}

int CpsFrame()
{
  int nDone,nTotal,nNext,i;

  if (CpsReset) DrvReset();

  CpsRwGetInp(); // Update the input port values

  nDone=0; nTotal=nCpsCycles;

  SekOpen(0);
  
  SekInterrupt(2); // VBlank
  for (i=0; i<4; i++)
  {
    int nStart,nEnd;
    nNext=((i+1)*nTotal) >> 2; // find out next cycle count to run to
    nDone+=SekRun(nNext-nDone); // run cpu

    CpsScodePoll(); // See which code has been passed
    // Run sound chips for this section
    nStart=i<<8; nEnd=(i+1)<<8; 
         if (Cps==2) QsndSectRun(nStart,nEnd);
    else if (Cps==1) PsndSectRun(nStart,nEnd);
  }
  SekClose();

  GetPalette(0,4); // Get palette
  CpsObjGet();     // Get objects
  memcpy(CpsSaveReg,CpsReg,0x100); // Registers correct now

  if (pBurnDraw==NULL) return 0; // Done

  // Draw the screen -----------------
  CpsDraw();
  return 0;
}
