#include "burnint.h"
// Music player

static int DrvInputInfo(struct BurnInputInfo *pii,unsigned int i)
{
  // Inputs common to all CPS2 games
  if (i>=sizeof(CpsFsi)/sizeof(CpsFsi[0])) return 1; // out of range
  if (pii!=NULL) *pii=CpsFsi[i];  return 0;
}

static int DrvInit()
{
  int nRet=0; int i=0; unsigned char *pqs=NULL;
  Cps=2;
  nCpsRomLen=  4*0x080000;
  nCpsCodeLen= 1*0x080000;
  nCpsGfxLen=    0x800000;
  nCpsZRomLen= 2*0x020000;
  nCpsQSamLen= 2*0x400000;
  nRet=CpsInit(); if (nRet!=0) return 1;

  // Load program roms (as they are on the roms) for CpsRom
  for (i=0;i<4;i++)
  { nRet=BurnLoadRom(CpsRom+0x080000*i,1+i,1); if (nRet!=0) return 1; }

  // Make decrypted rom
  memcpy(CpsCode,CpsRom,nCpsCodeLen);
  nRet=BurnXorRom(CpsCode,0,1); if (nRet!=0) return 1;

  // Others (not encrypted)
  for (i=1;i<4;i++)
  { nRet=BurnLoadRom(CpsRom+0x080000*i,1+i,1); if (nRet!=0) return 1; }

  // Load graphics roms
  Cps2LoadTiles(CpsGfx,5);

  // Load Z80 Roms
  nRet=BurnLoadRom(CpsZRom         , 9,1);
  nRet=BurnLoadRom(CpsZRom+ 0x20000,10,1);

  // Load Q Sample Roms
  pqs=(unsigned char *)CpsQSam;
  nRet=BurnLoadRom(pqs         ,11,1);
  nRet=BurnLoadRom(pqs+0x400000,12,1);
  BurnByteswap(pqs,nCpsQSamLen);

  nRet=CpsRunInit(); if (nRet!=0) return 1;
  // Ready to go
  return 0;
}

static int DrvExit()
{
  CpsRunExit();
  CpsExit();
  nCpsQSamLen=0;
  nCpsZRomLen=0;
  nCpsGfxLen=0;
  nCpsCodeLen=0;
  nCpsRomLen=0;
  Cps=0;
  return 0;
}

// --------------------- Different versions ------------------
static int DrvZipName(char **pszName,unsigned int i)
{
  if (i==0) { if (pszName!=NULL) *pszName="sfz.zip"; return 0; }
  if (i==1) { if (pszName!=NULL) *pszName="mshvsf.zip"; return 0; }
  return 1;
}

struct SfzRomDesc
{ char szName[16]; int nLen; unsigned int nCrc; int nType; };

// Street Fighter Zero Jpn -----------------------------
static struct SfzRomDesc SfzjRom[]=
{
  {"sfzux.03a",0x80000,0x1a3160ed, 0x10}, // xor decryption table
  {"sfzu.03a" ,0x80000,0x49fc7db9, 0x10}, // 68000 code (encrypted)
  {"sfz.04a"  ,0x80000,0x5f99e9a5, 0x10},
  {"sfz.05"   ,0x80000,0x0810544d, 0x10},
  {"sfz.06"   ,0x80000,0x806e8f38, 0x10}, 

  // graphics:
  {"sfz.14"  ,0x200000,0x90fefdb3,    1}, //  5
  {"sfz.16"  ,0x200000,0x5354c948,    1},
  {"sfz.18"  ,0x200000,0x41a1e790,    1},
  {"sfz.20"  ,0x200000,0xa549df98,    1},

  // sound - z80 roms
  {"mvs.01"  ,0x020000,0,    2}, //  9
  {"mvs.02"  ,0x020000,0,    2},
  // sound - samples
  {"mvs.11"  ,0x400000,0,    2}, // 11
  {"mvs.12"  ,0x400000,0,    2},
};

static int SfzjRomInfo(struct BurnRomInfo *pri,unsigned int i)
{
  if (i>=sizeof(SfzjRom)/sizeof(SfzjRom[0])) return 1;
  if (pri!=NULL)
  {
    pri->nLen =SfzjRom[i].nLen;
    pri->nCrc =SfzjRom[i].nCrc;
    pri->nType=SfzjRom[i].nType;
  }
  return 0;
}

static int SfzjRomName(char **pszName,unsigned int i,int nAka)
{
  if (i>=sizeof(SfzjRom)/sizeof(SfzjRom[0])) return 1;
  if (nAka!=0) return 1;
  *pszName=SfzjRom[i].szName;
  return 0;
}

struct BurnDriver BurnDrvCpsSfzMusicMshvsf=
{
  {"sfzmusicmshvsf"  ,"MSH Vs Street Fighter Music",""},
  DrvZipName, SfzjRomInfo, SfzjRomName, DrvInputInfo,
  DrvInit,DrvExit,CpsFrame,NULL,
  &CpsRecalcPal,384,224
};

// -----------------------------------------------------
