#include "burnint.h"
// System 16 - 'Ground' emulation

unsigned int *BsysPal=NULL; // Pointer to high-color palette (output format)
unsigned int nBsysPalLen=0; // number of colors, must be a power of two

// This function loads code roms. All even roms should be first, followed by all odd roms.
int BsysLoadCode(unsigned char *Rom,int nStart,int nCount)
{
  int i=0,nOdd=0; int nRet=0;
  nCount>>=1; // Half the count, then load all even then all odd

  for (nOdd=0;nOdd<2;nOdd++)
  {
    unsigned char *pLoad; pLoad=Rom+(nOdd^1); // ^1 for byteswapped

    for (i=0;i<nCount;i++)
    {
      struct BurnRomInfo ri;
      // Load these even/odd bytes
      nRet=BurnLoadRom(pLoad,nStart+i,2); if (nRet!=0) return 1;

      // Increment position by the length of the rom * 2
      ri.nLen=0; BurnDrvGetRomInfo(&ri,nStart+i); pLoad+=ri.nLen<<1;
    }
    nStart+=nCount; // Move on to the odd byte roms
  }

  return 0;
}

// Load 3 roms as bitplanes of tile data
int BsysLoadTiles(unsigned char *Tile,int nStart)
{
  struct BurnRomInfo ri; unsigned char *TileTmp=NULL;
  int i=0; int nRetVal=0;
  ri.nLen=0; BurnDrvGetRomInfo(&ri,nStart); // Find out how big the tile roms are

  // Load tile data (three bitfields)
  TileTmp=(unsigned char *)malloc(ri.nLen); if (TileTmp==NULL) return 1;
  memset(TileTmp,0,ri.nLen);
  for (i=0;i<3;i++)
  {
    int nRet;  nRet=BurnLoadRom(TileTmp,nStart+i,1);  if (nRet!=0) nRetVal=1;
    BurnLoadBitField(Tile,TileTmp,i,ri.nLen);
  }
  free(TileTmp); TileTmp=NULL;

  return nRetVal;
}

// Change color 15 to 0, to eliminate the System16 (guide?)lines
int BsysZeroSprites(unsigned char *Spr,int nLen)
{
  unsigned char *ps,*pEnd;
  if (nLen<=0) return 1;
  pEnd=Spr+nLen;
  ps=Spr;

  do
  {
    unsigned char x;
    x=(unsigned char)(~ps[0]);
    if ((x&0xf0)==0) ps[0]&=0x0f;
    if ((x&0x0f)==0) ps[0]&=0xf0;
    ps++;
  }
  while (ps<pEnd);

  return 0;
}

