#include <stdlib.h>
#include "cmc_int.h"

short *Precalc = NULL;

// ComaC : rotuine used to precalculate the table used for interpolation
void cmc_4p_Precalc(void)
{
  int a, x, x2, x3;

#ifndef MMX
  Precalc = (short *)malloc(sizeof(short) * 4096 * 4);
#else
  Precalc = &PrecalcMMX;
#endif

  for (a = 0; a < 4096; a++)
  {
    x = a * 4;			// x = 0..16384
    x2 = x * x / 16384;		// pow(x, 2);
    x3 = x2 * x / 16384;	// pow(x, 3);

    Precalc[a*4 +0] = -x/3 + x2/2 - x3/6;
    Precalc[a*4 +1] = -x/2 - x2   + x3/2 + 16384;
    Precalc[a*4 +2] =  x   + x2/2 - x3/2;
    Precalc[a*4 +3] = -x/6 + x3/6;
  }
}


#ifndef MMX
int ChannelMix_8S(int *Dest, int nLen,
		  char *Sample, int LoopEnd,
		  int *Pos,
		  int VolL, int VolR,
		  int LoopLen,
		  int IncPos)
{
  int p,sl,sr,l,r, slN, sl0, sl1, sl2, fp, Active=1;

  while (nLen--)
  {
    if (*Pos >= LoopEnd)
    {
      if (!LoopLen)
      {
        Active=0;
      }
      else
      {
        *Pos -= LoopLen;
      }
    }

    if (!Active) { *Pos=0; return 0; }

    // Quit if we reached the end of the sample

    p=(*Pos)>>12; // Find the current sample position
    fp = (*Pos) & ((1<<12) -1);
    
    slN=Sample[p-1]; // Get the sample
    sl0=Sample[p]; // Get the sample
    sl1=Sample[p+1]; // Get the sample
    sl2=Sample[p+2]; // Get the sample

    sl = (slN*Precalc[fp*4 +0] + sl0*Precalc[fp*4 +1] +
          sl1*Precalc[fp*4 +2] + sl2*Precalc[fp*4 +3]) / 64;//16384;

    sr=sl;
    
    sl*=VolL;
    sr*=VolR;

    // Add to the sound currently in the buffer
    Dest[0]+=sl; Dest[1]+=sr;
    Dest+=2;

    *Pos += IncPos;  // increment sample position based on pitch
  }

  return Active;
}

int ChannelMix_8U(int *Dest, int nLen,
		  unsigned char *Sample, int LoopEnd,
		  int *Pos,
		  int VolL, int VolR,
		  int LoopLen,
		  int IncPos)
{
  int p,sl,sr,l,r, slN, sl0, sl1, sl2, fp, Active=1;

  while (nLen--)
  {
    if (*Pos >= LoopEnd)
    {
      if (!LoopLen)
      {
        Active=0;
      }
      else
      {
        *Pos -= LoopLen;
      }
    }

    if (!Active) { *Pos=0; return 0; }

    // Quit if we reached the end of the sample

    p=(*Pos)>>12; // Find the current sample position
    fp = (*Pos) & ((1<<12) -1);
    
    slN=Sample[p-1]-0x80; // Get the sample
    sl0=Sample[p]-0x80; // Get the sample
    sl1=Sample[p+1]-0x80; // Get the sample
    sl2=Sample[p+2]-0x80; // Get the sample

    sl = (slN*Precalc[fp*4 +0] + sl0*Precalc[fp*4 +1] +
          sl1*Precalc[fp*4 +2] + sl2*Precalc[fp*4 +3]) / 64;//16384;

    sr=sl;
    
    sl*=VolL;
    sr*=VolR;

    // Add to the sound currently in the buffer
    Dest[0]+=sl; Dest[1]+=sr;
    Dest+=2;

    *Pos += IncPos;  // increment sample position based on pitch
  }

  return Active;
}

void AdaptSoundBuff(int *Src, short *Dest, int Len)
{

#define CLIP(A) (  (A) < -0x8000 ? -0x8000 : (A) > 0x7fff ? 0x7fff : (A)  )

  Len *= 2;
  while (Len--)
  {
    *Dest = CLIP( (*Src >> 8) );
    Src++;
    Dest++;
  }
}

void AdaptSoundBuff_Add(int *Src, short *Dest, int Len)
{

#define CLIP(A) (  (A) < -0x8000 ? -0x8000 : (A) > 0x7fff ? 0x7fff : (A)  )

  Len *= 2;
  while (Len--)
  {
    *Dest = CLIP( (*Src >> 8) + *Dest );
    Src++;
    Dest++;
  }
}

#endif
