// Doze - Dave's optimized Z80 emulator
// Code Maker - CB opcodes
#include "dam.h"

int DamcRotate(unsigned int op)
{
  int t=4,a=0,r=0;
  unsigned int nExt=0; // Extend code (00, dd or fd)
  static char szOp[8][4]={"rol","ror","rcl","rcr","sal","sar","sal","shr"};

  nExt=op>>8; if ((nExt&0xff)==0xcb) nExt=op>>16; // Get dd/fd extend code
  a=(op>>3)&7; r=op&7;

  DamOpStart(op);
  if (nExt)
  {
    // Get IX/IY offset (it's 2 bytes back because of the FD CB nn C6 opcode format)
    DamIxIyOffset(-2);
    t+=4;
  }
  if (nExt || r==6)
  {
    DamGetHL(nExt);
    t+=7;
  }
  DamGet8("dl",r,nExt);

  if (op>>8)
  {
    // One of the CB opcodes
    ot("\n");
    ot("  xor dh,dh\n");
    ot("  sahf\n");
    ot("  %s dl,1\n",szOp[a]);
    ot("  setc dh     ; ---- ---C\n");
    ot("  test dl,dl\n");
    ot("  lahf\n");
    ot("  and ah,0xc4 ; SZ-- -P--\n");
    ot("  or  ah,dh   ; SZ-- -P-C\n");
    ot("\n");
  }
  else
  {
    // One of the normal rotates
    ot("\n");
    ot("  sahf\n");
    ot("  %s dl,1\n",szOp[a]);
    ot("  setc dh     ; ---- ---C\n");
    ot("  and ah,0xc4 ; ..00 0.00\n");
    ot("  or  ah,dh   ; ..00 0.0C\n");
    ot("\n");
  }

  if (nExt || r==6) DamPut8("dl",6,nExt); // Put to memory
  if (r!=6) DamPut8("dl",r,0); // Put to register
  // (both of the above happen in the undocumented opcodes)
  DamOpDone(t,0);
  return op;
}

static int DamcBit(unsigned int op)
{
  int b=0,r=0,t=4;
  unsigned int nExt=0; // Extend code (00, dd or fd)

  nExt=op>>16; b=(op>>3)&7; r=op&7;

  DamOpStart(op);

  if (nExt)
  {
    // Get IX/IY offset (it's 2 bytes back because of the FD CB nn C6 opcode format)
    DamIxIyOffset(-2);
    t+=8;
  }
  if (nExt || r==6)
  {
    DamGetHL(nExt);
    t+=4;
  }

  DamGet8("dl",r,nExt);
  ot("  mov dh,ah\n");
  ot("  and dl,0x%.2x\n",1<<b);
  ot("  lahf\n");
  ot("  and dh,0x01 ; Preserve ---- ---C\n");
  ot("  and ah,0xc4 ;          SZ-- -P--\n");
  ot("  or  ah,dh   ;          SZ-- -P0C\n");
  ot("  or  ah,0x10 ;          SZ-1 -P0C\n");
  DamOpDone(t,0);
  return op;
}

static int DamcSet(unsigned int op)
{
  int b=0,r=0,t=4,s=0;
  unsigned int nExt=0; // Extend code (00, dd or fd)
  char *szTarg="";

  nExt=op>>16; b=(op>>3)&7; r=op&7; s=(op>>6)&1;

  DamOpStart(op);

  if (nExt)
  {
    // Get IX/IY offset (it's 2 bytes back because of the FD CB nn C6 opcode format)
    DamIxIyOffset(-2);
    t+=4;
  }
  if (nExt || r==6)
  {
    DamGetHL(nExt);
    t+=7;
  }

  if (nExt || r==6)
  {
    // Get to dl, then change it
    DamGet8("dl",6,nExt);
    szTarg="dl";
  }
  else
  {
    // Do it direct
    szTarg=DamLoc8(r,nExt);
  }

  if (s) ot("  or  %s,0x%.2x\n",szTarg,1<<b);
  else   ot("  and %s,0x%.2x\n",szTarg,0xff^(1<<b));

  if (nExt || r==6)
  {
    DamPut8("dl",6,nExt); // Put to memory
    if (r!=6) DamPut8("dl",r,0); // (undocumented opcodes also put to register)
  }

  DamOpDone(t,0);
  return op;
}

int DamcOpCb(unsigned int op)
{
  if ((op&0xc0)==0x00) return DamcRotate(op);
  if ((op&0xc0)==0x40) return DamcBit(op);
  if ((op&0x80)==0x80) return DamcSet(op);
  return 0x00; // nop
}
