#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "SDL.h"
#include "burn.h"
#include "defaults.h"
#include "config.h"
#ifdef __BEOS__
#include <OS.h>
#define usleep snooze
#endif

SDL_Surface *Pantalla;//, *Work;
int BPP, SDL_X, SDL_Y;
int SDL_Rate, SDL_TBuff;
int SDL_Rmask, SDL_Gmask, SDL_Bmask;
int SDL_Rshift, SDL_Gshift, SDL_Bshift;
int SDL_Rloss, SDL_Gloss, SDL_Bloss;

volatile int aWriteBuffer, aReadBuffer;
int aBuffSize;
short *AudioBuffers;

int nABuffers=def_nABuffers;


unsigned int SDL_HighCol(int R, int G, int B, int A)
{
  return  (  (R >> SDL_Rloss << SDL_Rshift) & SDL_Rmask  ) |
          (  (G >> SDL_Gloss << SDL_Gshift) & SDL_Gmask  ) |
          (  (B >> SDL_Bloss << SDL_Bshift) & SDL_Bmask  );
}


int VideoInit(void)
{
  if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO)) { return 1; }
//  SDL_EnableKeyRepeat(20,20);

  return 0;
}


int InitVideoMode(int X, int Y, int FullScreen, int *BytesPP)
{
  int Flags;
  int Color, r, g, b;
  char Titulo[256];
  SDL_Color Paleta[256];

  Flags = SDL_ANYFORMAT | SDL_HWPALETTE | SDL_HWSURFACE | SDL_DOUBLEBUF;
  if (FullScreen) { Flags |= SDL_FULLSCREEN; }

  Pantalla = SDL_SetVideoMode(X, Y, def_SDL_BPP, Flags);
  if (!Pantalla) { return 1; }

  BPP = Pantalla->format->BitsPerPixel;
  SDL_X = X;
  SDL_Y = Y;

  if (FullScreen) { SDL_ShowCursor(SDL_DISABLE); }

  sprintf(Titulo, "FinalBurn: %s", BurnDrvText(1));
  SDL_WM_SetCaption(Titulo,NULL);

  *BytesPP = Pantalla->format->BytesPerPixel;

  SDL_Rmask = Pantalla->format->Rmask;
  SDL_Gmask = Pantalla->format->Gmask;
  SDL_Bmask = Pantalla->format->Bmask;

  SDL_Rshift = Pantalla->format->Rshift;
  SDL_Gshift = Pantalla->format->Gshift;
  SDL_Bshift = Pantalla->format->Bshift;

  SDL_Rloss  = Pantalla->format->Rloss;
  SDL_Gloss  = Pantalla->format->Gloss;
  SDL_Bloss  = Pantalla->format->Bloss;

  if (BPP == 8)
  {
    // 256 color mode. Setup palete:
    Color = 0;
    for (b = 0; b < (1 << 2); b++)
    {
      for (g = 0; g < (1 << 3); g++)
      {
        for (r = 0; r < (1 << 3); r++)
        {
          Paleta[Color].r = r << 5;
          Paleta[Color].g = g << 5;
          Paleta[Color].b = b << 6;
          Color++;
        }
      }
    }

    SDL_SetColors(Pantalla, Paleta, 0, 256);

    SDL_Rmask = 7;
    SDL_Gmask = 7 << 3;
    SDL_Bmask = 3 << 6;

    SDL_Rshift = 0;
    SDL_Gshift = 3;
    SDL_Bshift = 6;

    SDL_Rloss = 5;
    SDL_Gloss = 5;
    SDL_Bloss = 6;
  }

  return BPP;
}


void *PrePaint(int *Pitch)
{
  if (SDL_MUSTLOCK(Pantalla)) { SDL_LockSurface(Pantalla); }
  *Pitch = Pantalla->pitch;
  return Pantalla->pixels;
}


void PostPaint(void)
{
}


int Paint(unsigned char *Src)
{
  if (SDL_MUSTLOCK(Pantalla)) { SDL_UnlockSurface(Pantalla); }
  SDL_Flip(Pantalla);

  return 0;
}


int VideoExit(void)
{
  SDL_Quit();
  return 0;
}


unsigned int mseconds(void)
{
  return SDL_GetTicks();
}


int ProcesInput(int *Value)
{
  SDL_Event Evt;
  int Exit = 0;

  if (SDL_PollEvent(&Evt))
  {
    switch (Evt.type)
    {
      case SDL_KEYDOWN :
      case SDL_KEYUP :
	*Value = Evt.key.state == SDL_PRESSED ? 1 : 0;
        return Evt.key.keysym.sym;
	break;
    }
  }

  return 0;
}


void AudioCallback(void *userdata, Uint8 *data, int Len)
{
  int Start, End;
  short *Dest = (short *)data;

  if ((aWriteBuffer - aReadBuffer) < (Len/4)) { aReadBuffer += Len/4; return; } // We have no data to supply

  Start = aReadBuffer % (nBurnSoundLen*nABuffers);
  End = (Start + (Len / 4)) % (nBurnSoundLen*nABuffers);

  if (End < Start)	// We cross the end of buffer cycle
  {
    memmove(Dest, AudioBuffers + Start*2,
	    (nBurnSoundLen*nABuffers - Start)*4);

    memmove(Dest + (nBurnSoundLen*nABuffers - Start)*2,
	    AudioBuffers,
	    End*4);
  }
  else			// We do not cross it
  {
    memmove(Dest, AudioBuffers + Start*2, Len);
  }

  aReadBuffer += Len/4;
}


int InitAudio(int SampleRate, int Divisor)
{
  SDL_AudioSpec Desired, Obtained;
  int BuffLen = SampleRate / Divisor;

  Desired.freq = SampleRate;
  Desired.format = AUDIO_S16LSB;
  Desired.channels = 2;
  Desired.samples = 1;

  // "samples" has to be power of two. Grrrr... stupid SDL...
  while ((Desired.samples << 1) < BuffLen)
  {
    Desired.samples <<= 1;
  }

  Desired.callback = AudioCallback;
  Desired.userdata = NULL;

  if (SDL_OpenAudio(&Desired, &Obtained)) { return 0; }

  if (Desired.samples != Obtained.samples)
  {
    printf("\nStupid SDL doesn't like my buffer size. :(\n\n");
    return 0;
  }

  SDL_Rate = Obtained.freq;
  SDL_TBuff = SDL_Rate / Divisor;

  AudioBuffers=(short *)malloc(nABuffers * 2 * sizeof(short) * SDL_TBuff);
  aWriteBuffer = 0;
  aReadBuffer = 0;
  aBuffSize = Obtained.samples;

  return SDL_Rate;
}


void PlayAudio(short *Buffer, int Len)
{
  int Start, End;

  // Wait until there is room for more audio data:
  while ((aWriteBuffer - aReadBuffer) > (SDL_TBuff*nABuffers-Len)) { usleep(200); }

  SDL_LockAudio();

  Start = aWriteBuffer % (SDL_TBuff*nABuffers);
  End = (Start + Len) % (SDL_TBuff*nABuffers);

  if (End < Start)
  {
    memmove(AudioBuffers + Start*2,
	    Buffer,
	    (SDL_TBuff*nABuffers - Start)*4);
    memmove(AudioBuffers,
	    Buffer + (SDL_TBuff*nABuffers - Start)*2,
	    End*4);
  }
  else
  {
    memmove(AudioBuffers + Start*2,
	    Buffer, Len*4);
  }

  aWriteBuffer += Len;

  SDL_UnlockAudio();
}


int WeAreLate(void)
{
  return (aWriteBuffer - aReadBuffer) < (SDL_TBuff*nABuffers/2);
}


void AudioExit(void)
{
  SDL_PauseAudio(1);
  SDL_CloseAudio();
}


void StartAudioPlay(void)
{
  SDL_PauseAudio(0);
}


void VideoSetPalete(unsigned char *Pal)
{
  SDL_Color Paleta[256];
  int a;

  for (a = 0; a < 256; a++)
  {
    Paleta[a].r = Pal[a*4];
    Paleta[a].g = Pal[a*4 +1];
    Paleta[a].b = Pal[a*4 +2];
  }

  SDL_SetColors(Pantalla, Paleta, 0, 256);
}
